@@ -1,19 +1,18 @@
 # Copyright (c) 2004 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
-# $Id: CNic.pm,v 1.27 2004/09/14 09:58:51 gavin Exp $
-
+# $Id: CNic.pm,v 1.31 2005/04/20 12:46:40 gavin Exp $
 package WWW::CNic;
-use vars qw($VERSION);
-$VERSION = '0.08';
-
 use LWP;
+use LWP::ConnCache;
 use HTTP::Request::Common;
 use URI::Escape;
-use Unicode::String qw(utf8);
 use Digest::MD5 qw(md5_hex);
+use vars qw($VERSION $CONNECTION_CACHE);
 use strict;
 
+our $VERSION = '0.13';
+
 =pod
 
 =head1 NAME
@@ -22,32 +21,27 @@ WWW::CNic - a web-based API for the CentralNic domain registry system.
 
 =head1 SYNOPSIS
 
-	#
-	# an example of a simple task performed through WWW::CNic:
-	#
-
 	use WWW::CNic;
 
 	my @suffixlist = qw(uk.com us.com de.com eu.com);
 
-	my $query = 	WWW::CNic->new(	command	=> 'search',
-					domain	=> 'toolkit-test',
-					use_ssl	=> 1,
-					timeout	=> 5
-			);
+	my $query = WWW::CNic->new(command => 'search', 'domain' => 'example');
 
 	$query->set(suffixlist => \@suffixlist);
 
-	my $response = $query->execute();
+	my $response = $query->execute;
+
+	if ($response->is_error) {
+		printf("Error: %s\n", $response->error);
 
-	if ($response->is_error()) {
-		print "ERROR: " . $response->error() . "\n";
 	} else {
 		foreach my $suffix(@suffixlist) {
-			if ($response->is_registered()) {
-				print "Domain $domain.$suffix is registered to ".$response->registrant($suffix)."\n";
+			if ($response->is_registered) {
+				printf("Domain %s.%s is registered to %s\n", $domain, $suffix, $response->registrant($suffix));
+
 			} else {
-				print "Domain $domain.$suffix is available for registration.\n";
+				printf("Domain %s.%s is available for registration\n", $domain, $suffix);
+
 			}
 		}
 	}
@@ -63,7 +57,7 @@ The design of C<WWW::CNic> is inspired greatly by C<LWP>, which is a prerequisit
 Installing C<WWW::CNic> is as simple as:
 
 	cd /usr/src
-	wget http://toolkit.centralnic.com/dist/perl/WWW-CNic-x.xx.tar.gz
+	wget http://toolkit.centralnic.com/dist/WWW-CNic-x.xx.tar.gz
 	tar zxvf WWW-CNic-x.xx.tar.gz
 	cd WWW-CNic-x.xx
 	perl Makefile.PL
@@ -80,11 +74,7 @@ C<LWP> - the WWW Library for Perl. This in turn requires C<libnet>, C<URI> and C
 
 =item 2
 
-either C<Crypt::SSLeay> or C<IO::Socket::SSL> for doing SSL transactions.
-
-=item 3
-
-C<Unicode::String>. This is for correcting the character set of domain names.
+An SSL toolkit (C<Crypt::SSLeay>, C<IO::Socket::SSL>) for doing HTTPS transactions.
 
 =item 4
 
@@ -114,25 +104,15 @@ C<password> - the password for your Reseller Handle.
 
 =item 2
 
-C<use_ssl> - when set to C<1>, causes all transactions to take place over a Secure Sockets Layer (SSL) connection. This is highly recommended for domain registration and modification transactions, although it's probably not needed for the other commands.
-
-In order to take advantage of this you need to have either the C<Crypt::SSLeay> or C<IO::Socket::SSL> modules installed. More information is available in the C<lwpcook> manpage.
+C<use_ssl> - when set to C<1>, causes all transactions to take place over a Secure Sockets Layer (SSL) connection. This is highly recommended for domain registration and modification transactions, although it's probably not needed for the other commands. In order to take advantage of this you need to have either the C<Crypt::SSLeay> or C<IO::Socket::SSL> modules installed. More information is available in the C<lwpcook> manpage.
 
 =item 3
 
-C<command> - required for every transaction. This may be one of the following:
-
-	suffixes
-	search
-	whois
-	register
-	register_atgb
-	modify
-	renewals
+C<command> - required for every transaction. This is a scalar containing the command name. The list of allowed commands is always growing, you should consult the Toolkit website for a complete list.
 
 =item 4
 
-C<domain> - only needed for domain registrations and modifications.
+C<domain> - only needed for domain registrations, modifications and other commands that act upon domains.
 
 =item 5
 
@@ -142,6 +122,10 @@ C<host> - allows you to use to a non-standard host. This is mainly useful for cl
 
 C<test> - when set to C<1>, this causes any domain registration and modification transactions to use the test database. Again, this is useful for testing and debugging.
 
+=item 7
+
+C<keep_alive> - by default, WWW::CNic will use C<LWP::ConnCache> to cache server connections. Setting C<keep_alive> to C<0> or C<undef> will turn this off.
+
 =back
 
 =head1 METHODS
@@ -178,13 +162,19 @@ L<WWW::CNic::Simple>
 
 =cut
 
+BEGIN {
+	our $CONNECTION_CACHE = LWP::ConnCache->new;
+}
+
 sub new {
 	# shift off the package name:
 	my $package = shift;
 	# convert to a hash:
 	my %args = @_;
 	# initialise the object:
-	my $self = {};
+	my $self = {
+		_test	=> 0,
+	};
 	foreach my $key(keys %args) {
 		$self->{"_$key"} = $args{$key};
 	}
@@ -192,7 +182,8 @@ sub new {
 	$self->{_agent} = LWP::UserAgent->new();
 	$self->{_agent}->timeout($self->{_timeout} || 10);
 	$self->{_agent}->agent(sprintf('%s/%s (LWP %s, Perl %vd, %s)', $package, $VERSION, $LWP::VERSION, $^V, ucfirst($^O)));
-	$self->{_base} = ($self->{_use_ssl} == 1 ? 'https' : 'http').'://'.($self->{_host} ne '' ? $self->{_host} : 'toolkit.centralnic.com').'/srv';
+	$self->{_agent}->conn_cache($CONNECTION_CACHE) unless (!$self->{_keep_alive});
+	$self->{_base} = (defined($self->{_use_ssl}) && $self->{_use_ssl} == 1 ? 'https' : 'http').'://'.(defined($self->{_host}) ? $self->{_host} : 'toolkit.centralnic.com').'/srv';
 	# bless into this package:
 	bless($self, $package);
 	return $self;
@@ -224,6 +215,10 @@ sub execute {
 		$self->{_command} eq 'get_pricing'	&& return $self->_get_pricing();
 		$self->{_command} eq 'delete'		&& return $self->_delete_domain();
 		$self->{_command} eq 'decline'		&& return $self->_decline_domain();
+		$self->{_command} eq 'undecline'	&& return $self->_undecline_domain();
+		$self->{_command} eq 'start_transfer'	&& return $self->_start_transfer();
+		$self->{_command} eq 'check_transfer'	&& return $self->_check_transfer();
+		$self->{_command} eq 'cancel_transfer'	&& return $self->_cancel_transfer();
 		die("Invalid command '$self->{_command}'");
 	}
 }
@@ -289,8 +284,6 @@ sub _register {
 	die("Missing domain name") if $self->{_domain} eq '';
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
-	# convert to latin1:
-	$self->{_domain} = utf8($self->{_domain})->latin1;
 	my ($domain, $suffix) = split(/\./, $self->{_domain}, 2);
 	my $chandle = (ref($self->{_params}{chandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}{chandle}) : $self->{_params}{chandle});
 	my $thandle = (ref($self->{_params}{thandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}{thandle}) : $self->{_params}{thandle});
@@ -489,7 +482,8 @@ sub _delete_domain {
 	my $self = shift;
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
-	die("Missing reason")   if $self->{_reason} eq '';
+	die("Missing reason")   if $self->{_reason}   eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
 	my %params = (
 				user		=> $self->{_username},
 				password	=> $self->_crypt_md5($self->{_password}),
@@ -508,6 +502,7 @@ sub _decline_domain {
 	my $self = shift;
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
 	my %params = (
 				user		=> $self->{_username},
 				password	=> $self->_crypt_md5($self->{_password}),
@@ -521,6 +516,82 @@ sub _decline_domain {
 	return WWW::CNic::Response::DeclineDomain->new($self->{_response}{_raw});
 }
 
+sub _undecline_domain {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	my $url = "$self->{_base}/undecline_domain";
+	my $req = POST($url, \%params);
+	$self->{_response}{_raw} = $self->_get($req);
+	use WWW::CNic::Response::UnDeclineDomain;
+	return WWW::CNic::Response::UnDeclineDomain->new($self->{_response}{_raw});
+}
+
+sub _start_transfer {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domains") if scalar(@{$self->{_params}{domains}}) < 1;
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+	);
+	my $i = 0;
+	foreach my $domain(@{$self->{_params}{domains}}) {
+		$params{"domains[$i]"} = $domain;
+		$i++;
+	}
+	my $url = "$self->{_base}/start_transfer";
+	my $req = POST($url, \%params);
+	$self->{_response}{_raw} = $self->_get($req);
+	use WWW::CNic::Response::StartTransfer;
+	return WWW::CNic::Response::StartTransfer->new($self->{_response}{_raw});
+}
+
+sub _check_transfer {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	my $url = "$self->{_base}/check_transfer";
+	my $req = POST($url, \%params);
+	$self->{_response}{_raw} = $self->_get($req);
+	use WWW::CNic::Response::CheckTransfer;
+	return WWW::CNic::Response::CheckTransfer->new($self->{_response}{_raw});
+}
+
+sub _cancel_transfer {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	my $url = "$self->{_base}/cancel_transfer";
+	my $req = POST($url, \%params);
+	$self->{_response}{_raw} = $self->_get($req);
+	use WWW::CNic::Response::CancelTransfer;
+	return WWW::CNic::Response::CancelTransfer->new($self->{_response}{_raw});
+}
+
 sub _get {
 	my ($self, $request) = @_;
 	my $response = $self->{_agent}->request($request);
